/*
 * TankMapViewerView.java
 */

package tankmapviewer;

import debug.Container;
import debug.SimpleObject;
import debug.SimpleTile;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.net.SocketAddress;
import java.util.ArrayList;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JPanel;
import org.omg.PortableInterceptor.SYSTEM_EXCEPTION;

/**
 * The application's main frame.
 */
public class TankMapViewerView extends FrameView {

   int SIZE_WIDTH = 550;
   int SIZE_HEIGHT = 550;
   int TILE_SIZE = 10;

    public TankMapViewerView(SingleFrameApplication app) {
        super(app);

        initComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = TankMapViewerApp.getApplication().getMainFrame();
            aboutBox = new TankMapViewerAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        TankMapViewerApp.getApplication().show(aboutBox);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        txtHost = new javax.swing.JTextField();
        lblHost = new javax.swing.JLabel();
        btnConnect = new javax.swing.JButton();
        pnlMap = new MapPanel();
        slidRefresh = new javax.swing.JSlider();
        lblRefresh = new javax.swing.JLabel();
        pnlObjects = new ObjectPanel();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();

        mainPanel.setName("mainPanel"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(tankmapviewer.TankMapViewerApp.class).getContext().getResourceMap(TankMapViewerView.class);
        txtHost.setText(resourceMap.getString("txtHost.text")); // NOI18N
        txtHost.setName("txtHost"); // NOI18N

        lblHost.setText(resourceMap.getString("lblHost.text")); // NOI18N
        lblHost.setName("lblHost"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(tankmapviewer.TankMapViewerApp.class).getContext().getActionMap(TankMapViewerView.class, this);
        btnConnect.setAction(actionMap.get("btnConnect_clicked")); // NOI18N
        btnConnect.setText(resourceMap.getString("btnConnect.text")); // NOI18N
        btnConnect.setName("btnConnect"); // NOI18N

        pnlMap.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0), 1, true));
        pnlMap.setName("pnlMap"); // NOI18N
        pnlMap.setPreferredSize(new java.awt.Dimension(800, 800));

        javax.swing.GroupLayout pnlMapLayout = new javax.swing.GroupLayout(pnlMap);
        pnlMap.setLayout(pnlMapLayout);
        pnlMapLayout.setHorizontalGroup(
            pnlMapLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 548, Short.MAX_VALUE)
        );
        pnlMapLayout.setVerticalGroup(
            pnlMapLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 548, Short.MAX_VALUE)
        );

        slidRefresh.setMaximum(5000);
        slidRefresh.setMinimum(20);
        slidRefresh.setValue(500);
        slidRefresh.setName("slidRefresh"); // NOI18N

        lblRefresh.setText(resourceMap.getString("lblRefresh.text")); // NOI18N
        lblRefresh.setName("lblRefresh"); // NOI18N

        pnlObjects.setBorder(new javax.swing.border.LineBorder(new java.awt.Color(0, 0, 0), 1, true));
        pnlObjects.setName("pnlObjects"); // NOI18N
        pnlObjects.setPreferredSize(new java.awt.Dimension(800, 800));

        javax.swing.GroupLayout pnlObjectsLayout = new javax.swing.GroupLayout(pnlObjects);
        pnlObjects.setLayout(pnlObjectsLayout);
        pnlObjectsLayout.setHorizontalGroup(
            pnlObjectsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 548, Short.MAX_VALUE)
        );
        pnlObjectsLayout.setVerticalGroup(
            pnlObjectsLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 548, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addComponent(pnlMap, javax.swing.GroupLayout.PREFERRED_SIZE, 550, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(pnlObjects, javax.swing.GroupLayout.PREFERRED_SIZE, 550, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGroup(mainPanelLayout.createSequentialGroup()
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lblHost)
                            .addComponent(txtHost, javax.swing.GroupLayout.PREFERRED_SIZE, 176, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(6, 6, 6)
                        .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(lblRefresh)
                            .addComponent(slidRefresh, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGap(18, 18, 18)
                        .addComponent(btnConnect)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(mainPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addGroup(mainPanelLayout.createSequentialGroup()
                            .addComponent(lblRefresh)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(slidRefresh, javax.swing.GroupLayout.PREFERRED_SIZE, 38, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addGroup(mainPanelLayout.createSequentialGroup()
                            .addComponent(lblHost)
                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                            .addComponent(txtHost, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(btnConnect))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(pnlObjects, javax.swing.GroupLayout.PREFERRED_SIZE, 550, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(pnlMap, javax.swing.GroupLayout.PREFERRED_SIZE, 550, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(28, Short.MAX_VALUE))
        );

        lblHost.getAccessibleContext().setAccessibleName(resourceMap.getString("jLabel1.AccessibleContext.accessibleName")); // NOI18N

        menuBar.setName("menuBar"); // NOI18N

        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 1130, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 944, Short.MAX_VALUE)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(statusAnimationLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3))
        );

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnConnect;
    private javax.swing.JLabel lblHost;
    private javax.swing.JLabel lblRefresh;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JPanel pnlMap;
    private javax.swing.JPanel pnlObjects;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JSlider slidRefresh;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private javax.swing.JTextField txtHost;
    // End of variables declaration//GEN-END:variables

    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;

    Image generateMapImgFromMap(ArrayList<SimpleTile> map){
        // Create a buffered image in which to draw
        BufferedImage bufferedImage = new BufferedImage(SIZE_WIDTH, SIZE_HEIGHT, BufferedImage.TYPE_INT_RGB);

        // Create a graphics contents on the buffered image
        Graphics2D g2d = bufferedImage.createGraphics();

        // Draw graphics
        g2d.setColor(Color.white);
        g2d.fillRect(0, 0, SIZE_WIDTH, SIZE_HEIGHT);


        //paint tiles
        for(SimpleTile tile: map){

            if(tile.isOutOfMap)
                g2d.setColor(Color.BLACK);
            else{
                if(!tile.isExplored){ //nicht exploriert
                    g2d.setColor(Color.GRAY);
                }
                else if(!tile.isPassable){//wenn nicht passable
                    if(tile.isWater)
                        g2d.setColor(Color.BLUE);
                    else
                        g2d.setColor(Color.RED);
                }
                else
                    g2d.setColor(Color.GREEN);
            }

            if((tile.x-1)*TILE_SIZE < SIZE_WIDTH && (tile.y-1)*TILE_SIZE < SIZE_HEIGHT)
                g2d.fillRect((tile.x-1)*TILE_SIZE, (tile.y-1)*TILE_SIZE, TILE_SIZE, TILE_SIZE);
        }


        // Graphics context no longer needed so dispose it
        g2d.dispose();


        return bufferedImage;
    }

    Image generateObjectMapImg(ArrayList<SimpleTile> map, ArrayList<SimpleObject> obj){
        // Create a buffered image in which to draw
        BufferedImage bufferedImage = new BufferedImage(SIZE_WIDTH, SIZE_HEIGHT, BufferedImage.TYPE_INT_RGB);

        // Create a graphics contents on the buffered image
        Graphics2D g2d = bufferedImage.createGraphics();

        // Draw graphics
        g2d.setColor(Color.white);
        g2d.fillRect(0, 0, SIZE_WIDTH, SIZE_HEIGHT);


        //paint tiles
        for(SimpleTile tile: map){

            if(tile.isOutOfMap)
                g2d.setColor(Color.BLACK);
            else{
                if(!tile.isExplored){ //nicht exploriert
                    g2d.setColor(new Color(75,75,75));
                }
                else if(!tile.isPassable){//wenn nicht passable
                    if(tile.isWater)
                        g2d.setColor(new Color(70,140,250));
                    else
                        g2d.setColor(new Color(50,50,50));
                }
                else
                    g2d.setColor(new Color(200,200,200));
            }

            if((tile.x-1)*TILE_SIZE < SIZE_WIDTH && (tile.y-1)*TILE_SIZE < SIZE_HEIGHT)
                g2d.fillRect((tile.x-1)*TILE_SIZE, (tile.y-1)*TILE_SIZE, TILE_SIZE, TILE_SIZE);
        }


        for(SimpleObject o: obj){
            switch(o.type){
                case 1: //Hangars
                    g2d.setColor(Color.RED);
                    break;
                case 2: //Tanks
                    if(o.durability>4000)
                        g2d.setColor(new Color(255,200,0));
                    else if(o.durability > 3000)
                        g2d.setColor(new Color(255,215,50));
                    else if(o.durability > 2000)
                        g2d.setColor(new Color(255,222,92));
                    else if(o.durability > 1000)
                        g2d.setColor(new Color(255,236,161));
                    break;
                case 3: //Crates
                    g2d.setColor(Color.cyan);
                    break;
            }

            if((o.x-1)*TILE_SIZE < SIZE_WIDTH && (o.y-1)*TILE_SIZE < SIZE_HEIGHT)
                g2d.fillRect((o.x-1)*TILE_SIZE, (o.y-1)*TILE_SIZE, TILE_SIZE, TILE_SIZE);

        }


        // Graphics context no longer needed so dispose it
        g2d.dispose();


        return bufferedImage;
    }


    class ObjectPanel extends JPanel
    {
        private Image img;
        public ObjectPanel()
        {
            img = generateMapImgFromMap(new ArrayList<SimpleTile>());
        }

        public void refreshImage(Container c){
            img = generateMapImgFromMap(c.map);
            this.repaint();
        }

        public void paint(Graphics g) {
            g.drawImage(img, 0,0, SIZE_WIDTH,SIZE_WIDTH, this);
        }
    }


    class MapPanel extends JPanel
    {
        private Image img;
        public MapPanel()
        {
            img = generateMapImgFromMap(new ArrayList<SimpleTile>());
        }

        public void refreshImage(Container c){
            img = generateObjectMapImg(c.map, c.objects);
            this.repaint();
        }
        
        public void paint(Graphics g) {
            g.drawImage(img, 0,0, SIZE_WIDTH,SIZE_WIDTH, this);
        }
    }

    @Action
    public void btnConnect_clicked() {

        Thread t = new Thread(new PollThread(new Long(slidRefresh.getValue())));
        t.start();
    }
   
    private class PollThread implements Runnable{

        Long pollTime = null;
        public PollThread(Long pollTime){
            this.pollTime = pollTime;
        }

        public void run() {
            int errors = 0;
            Socket sock = null;
            while(errors < 10){
                try {
                    Thread.sleep(this.pollTime);
                    sock = new Socket(txtHost.getText(), 6000);
                    OutputStream outStream = sock.getOutputStream();
                    InputStream inStream = sock.getInputStream();
                    ObjectInputStream objInStream = new ObjectInputStream(inStream);
                    ObjectOutputStream objOutStream = new ObjectOutputStream(outStream);

                    Container container = (Container) objInStream.readObject();
                    ((MapPanel) pnlMap).refreshImage(container);
                    ((ObjectPanel) pnlObjects).refreshImage(container);

                    objOutStream.flush();
                
                    sock.close();

                } catch (Exception ex) {
                   errors++;
                   ex.printStackTrace();
                }finally{
                   if(sock != null)
                    try {
                        sock.close();
                    } catch (IOException ex) {
                    }
                }
            }
        }
    }

}
